/**
    @file ft51_i2c_slave.h

    @brief Functions to control the FT51 I2C Slave.

    The FT51 has a single I2C slave peripheral, compliant with the
    published I2C specification 
    http://www.nxp.com/documents/user_manual/UM10204.pdf

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_I2C_SLAVE_H__
#define __FT51_I2C_SLAVE_H__

#include <stdint.h>
#include "ft51.h"
#include "ft51_dma.h"

/**
    @typedef I2C_slave_status
    @brief   Easy access to status flags.

**/
typedef struct I2C_slave_status I2C_slave_status;

/**
    @struct I2C_slave_status
    @brief Collection of I2C slave status values.
    
    **/
struct I2C_slave_status
{
    /** Slave addressed for write by master **/
    uint8_t rreq:1;
    /** Slave addressed for read by master **/
    uint8_t treq:1;
    /** Master has completed a receive operation **/
    uint8_t sendfin:1;
    /** Master has completed a transmit operation **/
    uint8_t recfin:1;
    /** Send, receive or address phase in progress, set and reset by START and STOP conditions **/
    uint8_t bus_active:1;
    /** Unused **/
    uint8_t unused1:1;
    /** Slave activated **/
    uint8_t da:1;
    /** Unused **/
    uint8_t unused2:1;
};

/**
    @brief   Call once, to initialise the slave and reset it to a known state.

    @param[in]  addr Slave address
**/                                  
void I2C_slave_initialise(uint8_t addr);

/**
    @brief   Read a specified number of bytes from the I2C Master.

    This transaction is orchestrated by the I2C Master.
    The number of bytes written may be less than the number
    requested if the master terminates the transaction early.
    @param[out] data            Caller-allocated buffer to receive any bytes 
                                read.
    @param[in]  number_to_read  Number of bytes to read.
    @return Number of bytes read from I2C Master.
**/
uint16_t I2C_slave_read(uint8_t *data, uint16_t number_to_read);

/**
    @brief   Write a specified number of bytes to an open device.

    This transaction is orchestrated by the I2C Master.
    The number of bytes written may be less than the number
    requested if the master terminates the transaction early.
    @param[in]  data             Buffer containing bytes to write.
    @param[in]  number_to_write  Number of bytes to read.
    @return Number of bytes written to I2C Master.
**/
uint16_t I2C_slave_write(const uint8_t *data, uint16_t number_to_write)

/**
    @brief   Determine I2C Slave status.

    @return Returns the status byte of the I2C Slave Status register
**/
uint8_t I2C_slave_get_status(void);

#endif // __FT51_I2C_SLAVE_H__