/**
    @file ft51_usb_internal.h

    @brief USB Library internal header file (not part of API).

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __USB_INTERNAL_H__
#define __USB_INTERNAL_H__

#include <stdint.h>

#include "ft51.h"
#include "ft51_usb.h"

#define USB_MAX_ENDPOINT_COUNT_CONTROL 2
#define USB_MAX_ENDPOINT_COUNT_0_TO_2 6
#define USB_MAX_ENDPOINT_COUNT_0_TO_6 14
#define USB_MAX_ENDPOINT_COUNT_0_TO_7 16

#ifndef USB_MAX_ENDPOINT_COUNT
/**
 @def USB_MAX_ENDPOINT_COUNT
 @brief The maximum number of endpoints which can be used in
 the library. The default value is for 2 control endpoints
 and an additional 4 BULK, INTERRUPT or ISOCHRONOUS
 endpoints. The more endpoints defined, the larger the
 interrupt code in USB_ISR() and the more memory required
 for the USB_ep structure.
 The following compiler flag will change the number of
 endpoints used:
 sdcc -D USB_MAX_ENDPOINT_COUNT=8
 */
#define USB_MAX_ENDPOINT_COUNT USB_MAX_ENDPOINT_COUNT_0_TO_2
#endif

#define SELECT_ENDPOINT                 0x00
#define READ_LAST_TRANSACTION_STATUS    0x40
#define SET_ENDPOINT_STATUS             0x40
#define READ_ENDPOINT_STATUS            0x80
#define SET_ENDPOINT_CONFIGURATION      0xB0
#define SET_ADDRESS_EN                  0xD0
#define SET_ENDPOINT_ENABLE             0xD8
#define SET_LPM_ENABLE                  0xEE
#define READ_LPM_STATUS                 0xEF
#define RD_BUFFER                       0xF0
#define WR_BUFFER                       0xF0
#define ACK_SETUP                       0xF1
#define CLR_BUFFER                      0xF2
#define SET_MODE                        0xF3
#define VALIDATE_BUFFER                 0xFA
#define RD_INTERRUPT_REG                0xF4
#define READ_CURRENT_FRAME_NUMBER       0xF5
#define SEND_RESUME                     0xF6


/** Bit 0 of byte returned from SELECT_ENDPOINT command **/
#define SELECT_BUFFER_FULL 0x01

/**
 @enum  USB_EP_NUMS
 @brief Relates to the USB ctx that holds pointers to the relative
 endpoints.
 **/
typedef enum
{
    CTRL_OUT, // Host to device.
    CTRL_IN,  // Device to host.
    EP_1_OUT,
    EP_1_IN,
    EP_2_OUT,
    EP_2_IN,
    EP_3_OUT,
    EP_3_IN,
    EP_4_OUT,
    EP_4_IN,
    EP_5_OUT,
    EP_5_IN,
    EP_6_OUT,
    EP_6_IN,
    EP_7_OUT,
    EP_7_IN,
} USB_EP_NUMS;

// Section 6.3.1 FT122 Data Sheet
typedef enum
{
    CTRL_OUT_INT = 1,
    CTRL_IN_INT = 2,
    EP_1_OUT_INT = 4,
    EP_1_IN_INT = 8,
    EP_2_OUT_INT = 16,
    EP_2_IN_INT = 32,
    BUS_RESET_INT = 64,
    SUSPEND_CHANGE_INT = 128,
    DMA_EOT_INT = 1,
    LPM_CHANGE_INT = 0x02,
    EP_3_OUT_INT = 1,
    EP_3_IN_INT = 2,
    EP_4_OUT_INT = 4,
    EP_4_IN_INT = 8,
    EP_5_OUT_INT = 16,
    EP_5_IN_INT = 32,
    EP_6_OUT_INT = 64,
    EP_6_IN_INT = 128,
    EP_7_OUT_INT = 1,
    EP_7_IN_INT = 2,
} INT_REGISTER;

/**
 @name READ_ENDPOINT_STATUS
 @brief Bitmap for value returned by Read of Endpoint Status Mask
 **/
//@{
#define SETUP_PACKET         0x04
#define BUFFER_0_FULL        0x20
#define BUFFER_1_FULL        0x40
#define ENDPOINT_STALLED     0x80
//@}

/**
 @name READ_LAST_TRANSACTION_STATUS
 @brief Bitmap for value returned by Read of Last Transaction Status
 **/
//@{
#define DATA_RX_TX_SUCCESS   0x01
#define ERROR_CODE           0x1e
#define SETUP_PACKET_LAST_TX 0x20
#define DATA_PACKET          0x40
#define STATUS_NOT_READ      0x80
//@}

/**
 @name SELECT_ENDPOINT
 @brief Bitmap of optional data byte returned by Read after Select Endpoint
 **/
//@{
#define USB_SELECT_ENDPOINT_BUFFER_FULL   0x01
#define USB_SELECT_ENDPOINT_STALLED       0x02
//@}

// Declarations for internally referenced functions
void USB_ISR(const uint8_t flags);

// Variables defined in usb_initialise.c, but visible throughout USB driver:

/** USB Hub configuration settings. **/
extern uint8_t USB_hub_config;
/** Current USB state. **/
extern USB_STATE USB_current_state;
/** Suspend/resume status changed flag. **/
extern volatile uint8_t USB_suspend_changed;
/** Reset activated flag. **/
extern volatile uint8_t USB_reset_signal;
/** Device in suspend state allowing power save mode to be entered. **/
extern volatile int8_t USB_core_suspend_request;
/** Current USB configuration: Set by SET_CONFIGURATION request. **/
extern uint8_t USB_configuration;
/* extern uint8_t                 USB_remote_wakeup;*/
/** Standard request callback **/
extern USB_request_callback USB_standard_req_cb;
/** Class request callback **/
extern USB_request_callback USB_class_req_cb;
/** Vendor request callback **/
extern USB_request_callback USB_vendor_req_cb;
/** Callback when the device resumes **/
extern USB_suspend_callback USB_resume_cb;
/** Callback when the device suspends **/
extern USB_suspend_callback USB_suspend_cb;
/** Callback when the host initiates a reset **/
extern USB_reset_callback USB_reset_cb;
/** Array of pointers to device endpoints **/
extern __xdata USB_endpoint USB_ep[USB_MAX_ENDPOINT_COUNT];
/** Decode of USB_ENDPOINT_SIZE definitions into actual endpoint
 *  Max Packet size values. **/
extern __code  const uint16_t USB_ep_size_bytes[12];
/** Most-recent Setup packet **/
extern __xdata uint8_t USB_received_setup_packet[8];
extern volatile uint8_t USB_LPM_changed;

#endif // __USB_INTERNAL_H__